'use strict';

(function init() {
  try {
    initEnv();
    initInjection();
    initFirebase();
    initMessageListeners();
    onInstalled();
    onUninstalled();
  } catch (error) {
    log.error('BACKGROUND ERROR', error)
    const action = 'error';
    const clientId = getClientId()
    const parsedError = JSON.stringify(error, Object.getOwnPropertyNames(error));
    parsedError = parsedError.substr(0, 999)
    const value = JSON.stringify({ page: 'background', error: parsedError})
    createAction({ action, clientId, value })
  }
})()

function initMessageListeners() {
  chrome.runtime.onMessage.addListener(async (request, sender, sendResponse) => {
    if (request.type == "googleLogin") {
      let result = await googleLogin()
      let user = result.user;
    } else if (request.type == "facebookLogin") {
      let result = await facebookLogin()
      let user = result.user;
    } else if (request.type == "twitterLogin") {
      let result = await twitterLogin()
      let user = result.user;
    } else if (request.type == "logout") {
      signOut()
    } else if (request.type == "screenCapture") {
      // let imageData = await screenCapture(request.imageRect);
      // chrome.tabs.sendMessage(sender.tab.id, { type: 'screenCapture', imageData });
    } else if (request.type == 'bug_report_screenshot') {
      // let imageData = await screenCapture();
      // let imageUrl = await uploadImage(imageData, `bug_report_${Date.now()}`)
    } else if (request.type == "downloadImage") {
      // downloadImage(request.imageData)
    } else if (request.type == "uploadImage") {
      // let imageUrl = await uploadImage(request.imageData, request.filename)
      // chrome.tabs.sendMessage(sender.tab.id, { type: 'imageDownloadUrl', imageUrl });
    } else if (request.type == "getPartnerList") {
      let partners = await getPartnerList()
      chrome.tabs.sendMessage(sender.tab.id, { type: 'getPartnerList', partners });
    } else if (request.type === 'orphanCheck') {
      sendResponse('good')
    } else {
      log.error('unknown message type')
    }
  });
}

function signOut() {
  log.debug('call signOut')
  firebase.auth().signOut().then(() => {
    log.debug('signout successful.')
    chrome.storage.local.set({ user: JSON.stringify({}) });
  }).catch((error) => {
    log.error('Sign-out Failed', error)
  });
}

async function twitterLogin() {
  log.debug('call twitterLogin');

  try {
    let provider = new firebase.auth.TwitterAuthProvider();
    let result = await firebase.auth().signInWithPopup(provider)
    log.debug('twitterLogin RESULT')
    return result
  } catch (error) {
    log.error('twitterLogin ERROR', error)
    if (error.code === 'auth/user-disabled') {
      alert("You're banned from Dmooji because you may have violated our terms of service.")
      throw error
    } else if (error.code === 'auth/account-exists-with-different-credential') {
      alert("Please login with your Google account.")
      throw error
    } else {
      throw error
    }
  }
}

async function facebookLogin() {
  log.debug('call facebookLogin');

  try {
    let provider = new firebase.auth.FacebookAuthProvider();
    provider.addScope('public_profile');
    provider.addScope('email');
    let result = await firebase.auth().signInWithPopup(provider)
    log.debug('facebookLogin RESULT')
    return result
  } catch (error) {
    log.error('facebookLogin ERROR', error)
    if (error.code === 'auth/user-disabled') {
      alert("You're banned from Dmooji because you may have violated our terms of service.")
      throw error
    } else if (error.code === 'auth/account-exists-with-different-credential') {
      alert("Please login with your Google account.")
      throw error
    } else {
      throw error
    }
  }
}

async function googleLogin() {
  log.debug('call googleLogin');

  try {
    let provider = new firebase.auth.GoogleAuthProvider();
    // provider.addScope('profile');
    provider.addScope('email');
    let result = await firebase.auth().signInWithPopup(provider)
    log.debug('googleLogin RESULT')
    return result
  } catch (error) {
    log.error('googleLogin ERROR', error)
    if (error.code === 'auth/user-disabled') {
      alert("You're banned from Dmooji because you may have violated our terms of service.")
    } else {
      throw error
    } 
  }
}

function initFirebase() {
  log.debug('call initFirebase')
  const config = {
    apiKey: "AIzaSyAiIpdPem7dRUUWPDHGOXG1eunSzzl11TY",
    authDomain: "dmooji-211117.firebaseapp.com",
    databaseURL: "https://dmooji-211117.firebaseio.com",
    projectId: "dmooji-211117",
    storageBucket: 'dmooji-211117.appspot.com',
    messagingSenderId: "636659387496"
  };
  firebase.initializeApp(config);
  firebase.auth().onAuthStateChanged(async (user) => {
    if (user) {
      log.debug('user login')
      let loginUser = {
        avatar: user.photoURL,
        email: user.email || user.uid, // sometimes email is null, then use uid as id
        name: user.displayName,
      }
      loginUser = await createUser(loginUser)
      chrome.storage.local.set({ user: JSON.stringify(loginUser) });
    } else {
      log.debug('User is not login')
      chrome.storage.local.set({ user: JSON.stringify({}) });
    }
  });
}

async function initEnv() {
  const NODE_ENV = await getEnv();
  if (NODE_ENV === 'development') {
    chrome.storage.local.set({
      dmoojiEnv: NODE_ENV,
    }, () => {
      log.enableAll()
      log.debug(NODE_ENV)
    });
  } else if (NODE_ENV === 'production') {
    chrome.storage.local.set({
      dmoojiEnv: NODE_ENV,
    }, () => {
      log.debug(NODE_ENV)
      log.disableAll()
    });
  }
}

function initInjection() {
  chrome.tabs.onUpdated.addListener((tabId, changeInfo, tab) => {
    if (tab.url && tab.url.includes('www.youtube.com/watch')) {
      if (changeInfo.status === "complete") {
        log.debug('tab complete')
        // call when next video
        chrome.tabs.executeScript(tabId, { code: 'window.reloadVideo && window.reloadVideo()' });
        chrome.storage.local.set({ tabId: tabId });

        // make sure scripts only injected once
        chrome.tabs.sendMessage(tabId, { inject_message: "are_you_there_content_script?" }, (msg) => {
          msg = msg || {};
          if (msg.status !== 'yes') {
            chrome.tabs.executeScript(tabId, { file: 'chrome/google-analytics-bundle.js' });
            chrome.tabs.executeScript(tabId, { file: 'chrome/cpa.js' });
            injectReactComponent('inject', tabId, () => log.debug('injection finished!'));
          }
        });
      }
    }
  });
}

async function injectReactComponent(name, tabId, next) {
  log.debug('call injectReactComponent')
  const NODE_ENV = await getEnv()

  if (NODE_ENV === 'production') {
    chrome.tabs.executeScript(tabId, { file: `/static/js/${name}.bundle.js` }, next);
    chrome.tabs.insertCSS(tabId, { file: `/static/css/${name}.bundle.css` }, next)
  } else {
    fetch(`http://localhost:3000/static/js/${name}.bundle.js`)
    .then(res => res.text())
    .then((fetchRes) => {
      chrome.tabs.executeScript(tabId, { code: fetchRes }, next);
    })
    .catch((error) => {
      log.error('executeScript error', error)
    })
  }
}

function onInstalled() {
  chrome.runtime.onInstalled.addListener(async (details) => {
    if (details.reason === 'install') {
      // open options page 
      log.debug('install')
      window.open('options.html', '_blank');
      let lan = window.navigator.languages[0];
      // log install event
      try {
        const clientId = getClientId()
        createAction({ action: 'install', clientId: clientId, value: lan })
        .then(() => {})
        .catch(() => {})
        dmoojiSendGaEvent('install')
      } catch (error) {
        log.error(error)
      }
    } else if (details.reason === 'update') {
      log.debug('update')
      reinjectScript()
    } else {
      log.debug('other', details)
    }
  })
}

function reinjectScript() {
  chrome.storage.local.get(['tabId'], async (result) => {
    let tabId = JSON.parse(result.tabId || null);
    if (tabId) {
      log.debug('call reinjectScript')
      chrome.tabs.executeScript(tabId, { code: "window.location.reload()" });
    }
  });
}

async function onUninstalled() {
  const NODE_ENV = await getEnv()
  const APIHOST = NODE_ENV === 'production' ? 'https://api.dmooji.tv' : 'http://localhost:5000';
  const version = chrome.runtime.getManifest().version;
  const clientId = getClientId()
  const lan = window.navigator.languages[0]
  chrome.runtime.setUninstallURL(`${APIHOST}/api/v3/actions/uninstall?version=${version}&id=${clientId}&lan=${lan}`);
}

// async function screenCapture(imageRect) {
//   log.debug('call screenCapture')
//   return new Promise((resolve, reject) => {
//     chrome.tabs.captureVisibleTab(null, {format: 'png'}, async (dataURI) => {
//       if (dataURI) {
//         if (imageRect) {
//           let imageData = await cropImage(dataURI, imageRect)
//           resolve(imageData)
//         } else {
//           resolve(dataURI)
//         }
//       }
//     })
//   })
// }

// async function cropImage(dataURI, imageRect) {
//   log.debug('call cropImage')
//   return new Promise((resolve, reject) => {
//     let image = new Image();
//     image.src = dataURI;
//     image.onload = async () => {
//       let imageData = await cropImageHelper(image, imageRect);
//       resolve(imageData)
//     };
//   })
// }

// async function cropImageHelper(imgObj, imageRect) {
//   log.debug('call cropImageHelper')
//   // retina ratio is 2 and non-retina ratio is 1
//   let ratio = window.devicePixelRatio;
//   // crop image with canvas
//   let canvas = document.createElement('canvas');
//   canvas.width = imageRect.width;
//   canvas.height = imageRect.height;
//   let ctx = canvas.getContext('2d');
//   ctx.drawImage(imgObj, imageRect.x*ratio, imageRect.y*ratio, imageRect.width*ratio, imageRect.height*ratio, 0, 0, imageRect.width, imageRect.height);

//   // add watermark
//   let wwidth = 500/4;
//   let wheight = 250/4;
//   let watermark = await getWatermark()
//   ctx.drawImage(watermark, imageRect.width-wwidth-15, imageRect.height-wheight-40, wwidth, wheight);
//   return canvas.toDataURL();
// }

// function getWatermark() {
//   log.debug('call getWatermark')
//   return new Promise((resolve, reject) => {
//     fetch("https://s3-us-west-1.amazonaws.com/dmooji-image/watermark.png")
//     .then(res => res.arrayBuffer())
//     .then((buffer) => {
//       let base64Flag = 'data:image/png;base64,';
//       let watermark_uri = arrayBufferToBase64(buffer);
//       let watermark = new Image()
//       watermark.src = base64Flag + watermark_uri;
//       watermark.onload = () => {
//         resolve(watermark)
//       };
//     })
//   })
// }

// function arrayBufferToBase64(buffer) {
//   var binary = '';
//   var bytes = [].slice.call(new Uint8Array(buffer));
//   bytes.forEach((b) => binary += String.fromCharCode(b));
//   return window.btoa(binary);
// };

// function uploadImage(imageData, filename) {
//   log.debug('call uploadImage')

//   return new Promise((resolve, reject) => {
//     // convert from base64 to blob 
//     fetch(imageData)
//     .then(res => res.blob())
//     .then((blob) => {
//       let metadata = { 'contentType': 'image/png' };
//       let storageRef = firebase.storage().ref();
//       storageRef.child(`dmooji_share/${filename}.png`).put(blob, metadata).then((snapshot) => {
//         snapshot.ref.getDownloadURL().then((url) => {
//           log.debug('File available at', url);
//           resolve(url)
//         });
//       })
//     })
//   })
// }

// function downloadImage(dataURI) {
//   log.debug('call downloadImage')
//   fetch(dataURI)
//   .then(res => res.blob())
//   .then((blob) => {
//     let url = URL.createObjectURL(blob);
//     chrome.downloads.download({
//       url: url,
//       filename: 'dmooji_image_share.png'
//     });
//   })
// }

async function getPartnerList() {
  log.debug('call getPartnerList')
  let db = firebase.firestore();
  db.settings({ timestampsInSnapshots: true });
  let snapshot = await db.collection("partners").orderBy("createdAt").get()
  let results = []
  snapshot.forEach((doc) => {
    results.push(doc.data());
  });
  return results;
}
