
function getEnv() {
  return new Promise((resolve, reject) => {
    chrome.management.getSelf((result) => {
      if (result.installType === 'development') {
        resolve('development')
      } else if (result.installType === 'normal') {
        resolve('production');
      }
    })
  })
}

async function xhrHandler({ method, url, name, body }) {
  console.log(`call ${name}`);

  return new Promise((resolve, reject) => {

    if (!method) reject('method missing');
    if (!name) reject('name missing');
    if (!url) reject('url missing');

    const xhr = new XMLHttpRequest();
    
    try {
      xhr.open(method, url, true);
      xhr.setRequestHeader('Content-type','application/json; charset=utf-8');
      xhr.responseType = 'json';

      xhr.onload = () => {
        if (xhr.readyState === 4 && xhr.status === 200) {
          let response = xhr.response;
          console.log(`${name} response`, response)
          resolve(response)
        } else if (xhr.readyState === 4 && xhr.status === 400) {
          let response = xhr.response;
          console.error(`${name} error`, response)
          reject(response);
        } else {
          console.error(`${name} error`)
          reject();
        }
      };

      xhr.onerror = () => {
        console.error(`${name} onerror`)
        reject();
      };

      if (method.toLowerCase() === 'post') {
        if (!body) reject('body missing');
        const json = JSON.stringify(body);
        xhr.send(json);
      } else {
        xhr.send();
      }
      
    } catch (error) {
      console.error(`${name} catch error`, xhr.statusText)
      reject(error);
    }
  });
}

async function createUser(body) {
  const NODE_ENV = await getEnv();
  const APIHOST = NODE_ENV === 'production' ? 'https://api.dmooji.tv' : 'http://localhost:5000';

  try {
    const clientId = await getClientId()
    body.clientId = clientId;
  } catch (error) {
    console.error(error)
  }

  try {
    let response = await xhrHandler({
      method: 'POST',
      url: `${APIHOST}/api/v3/users/new`,
      name: 'createUser',
      body: body,
    })
    return response.data
  } catch (error) {
    throw error
  }
}

async function createAction(body) {
  const NODE_ENV = await getEnv();
  const APIHOST = NODE_ENV === 'production' ? 'https://api.dmooji.tv' : 'http://localhost:5000';

  try {
    let response = await xhrHandler({
      method: 'POST',
      url: `${APIHOST}/api/v3/actions/new`,
      name: 'createAction',
      body: body,
    })
    return response.data
  } catch (error) {
    throw error
  }
}
